@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/main.c,v 1.57 2009/12/16 09:35:41 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/main.c,v 1.61 2010/02/13 11:23:55 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2009	PgPool Global Development Group
+ * Copyright (c) 2003-2010	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -266,6 +266,12 @@ int main(int argc, char **argv)
 		}
 	}
 
+#ifdef USE_SSL
+	/* global ssl init */
+	SSL_library_init();
+	SSL_load_error_strings();
+#endif /* USE_SSL */
+
 	mypid = getpid();
 
 	if (pool_init_config())
@@ -790,6 +796,7 @@ static int read_status_file(void)
 	FILE *fd;
 	char fnamebuf[POOLMAXPATHLEN];
 	int i;
+	bool someone_wakeup = false;
 
 	snprintf(fnamebuf, sizeof(fnamebuf), "%s/%s", pool_config->logdir, STATUS_FILE_NAME);
 	fd = fopen(fnamebuf, "r");
@@ -798,7 +805,7 @@ static int read_status_file(void)
 		pool_log("Backend status file %s does not exist", fnamebuf);
 		return -1;
 	}
-	if (fread(&backend_rec, 1, sizeof(backend_rec), fd) <= 0)
+	if (fread(&backend_rec, 1, sizeof(backend_rec), fd) != sizeof(backend_rec))
 	{
 		pool_error("Could not read backend status file as %s. reason: %s",
 				   fnamebuf, strerror(errno));
@@ -812,7 +819,21 @@ static int read_status_file(void)
 		if (backend_rec.status[i] == CON_DOWN)
 			BACKEND_INFO(i).backend_status = CON_DOWN;
 		else
+		{
 			BACKEND_INFO(i).backend_status = CON_CONNECT_WAIT;
+			someone_wakeup = true;
+		}
+	}
+
+	/*
+	 * If no one woke up, we regard the status file bogus
+	 */
+	if (someone_wakeup == false)
+	{
+		for (i=0;i< pool_config->backend_desc->num_backends;i++)
+		{
+			BACKEND_INFO(i).backend_status = CON_CONNECT_WAIT;
+		}
 	}
 
 	return 0;
@@ -842,7 +863,7 @@ static int write_status_file(void)
 		backend_rec.status[i] = BACKEND_INFO(i).backend_status;
 	}
 
-	if (fwrite(&backend_rec, 1, sizeof(backend_rec), fd) <= 0)
+	if (fwrite(&backend_rec, 1, sizeof(backend_rec), fd) != sizeof(backend_rec))
 	{
 		pool_error("Could not write backend status file as %s. reason: %s",
 				   fnamebuf, strerror(errno));
@@ -1476,6 +1497,8 @@ int health_check(void)
 {
 	int fd;
 	int sts;
+	static bool is_first = true;
+	static char *dbname;
 
 	/* V2 startup packet */
 	typedef struct {
@@ -1486,13 +1509,21 @@ int health_check(void)
 	char kind;
 	int i;
 
+	/* Do health check during recovery */
 	if (*InRecovery)
 		return 0;
 
+ Retry:
+	/*
+	 * First we try with "postgres" database.
+	 */
+	if (is_first)
+		dbname = "postgres";
+
 	memset(&mysp, 0, sizeof(mysp));
 	mysp.len = htonl(296);
 	mysp.sp.protoVersion = htonl(PROTO_MAJOR_V2 << 16);
-	strcpy(mysp.sp.database, "template1");
+	strcpy(mysp.sp.database, dbname);
 	strncpy(mysp.sp.user, pool_config->health_check_user, sizeof(mysp.sp.user) - 1);
 	*mysp.sp.options = '\0';
 	*mysp.sp.unused = '\0';
@@ -1554,12 +1585,24 @@ int health_check(void)
 			return i+1;
 		}
 
+		if (is_first)
+			is_first = false;
+
 		/*
 		 * If a backend raised a FATAL error(max connections error or
 		 * starting up error?), do not send a Terminate message.
 		 */
 		if ((kind != 'E') && (write(fd, "X", 1) < 0))
 		{
+			if (!strcmp(dbname, "postgres"))
+			{
+				/*
+				 * Retry with template1
+				 */
+				dbname = "template1";
+				goto Retry;
+			}
+
 			pool_error("health check failed during write. host %s at port %d is down. reason: %s. Perhaps wrong health check user?",
 					   BACKEND_INFO(i).backend_hostname,
 					   BACKEND_INFO(i).backend_port,
