@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pool_proto_modules.c,v 1.28 2009/12/06 07:52:49 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_proto_modules.c,v 1.36 2010/02/22 03:53:11 t-ishii Exp $
  * 
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2009	PgPool Global Development Group
+ * Copyright (c) 2003-2010	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -351,7 +351,8 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 			IsA(node, VariableSetStmt) || IsA(node, DiscardStmt))
 		{
 			/*
-			 * PREPARE, DEALLOCATE and SET statements must be replicated.
+			 * PREPARE, SET, DEALLOCATE and DISCARD statements must be
+			 * replicated even if we are in master/slave mode.
 			 */
 			if (MASTER_SLAVE && TSTATE(backend) != 'E')
 				force_replication = 1;
@@ -598,6 +599,16 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 						free_parser();
 						return POOL_END;
 					}
+
+					/*
+					 * Check if some error detected.  If so, emit
+					 * log. This is usefull when invalid encoding error
+					 * occurs. In this case, PostgreSQL does not report
+					 * what statement caused that error and make users
+					 * confused.
+					 */
+					per_node_error_log(backend, i, string, "SimpleQuery: Error or notice message from backend: ", true);
+
 				}
 				if (commit)
 				{
@@ -642,6 +653,17 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 				string = POOL_ERROR_QUERY;
 				len = strlen(string) + 1;
 			}
+			else
+			{
+				/*
+				 * Check if some error detected.  If so, emit
+				 * log. This is usefull when invalid encoding error
+				 * occurs. In this case, PostgreSQL does not report
+				 * what statement caused that error and make users
+				 * confused.
+				 */
+				per_node_error_log(backend, MASTER_NODE_ID, string, "SimpleQuery: Error or notice message from backend: ", true);
+			}
 		}
 
 		/* send query to other than master nodes */
@@ -678,6 +700,16 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 				free_parser();
 				return POOL_END;
 			}
+
+			/*
+			 * Check if some error detected.  If so, emit
+			 * log. This is usefull when invalid encoding error
+			 * occurs. In this case, PostgreSQL does not report
+			 1* what statement caused that error and make users
+			 * confused.
+			 */
+			per_node_error_log(backend, i, string, "SimpleQuery: Error or notice message from backend: ", true);
+
 		}
 
 		/* send "COMMIT" or "ROLLBACK" to only master node if query is "COMMIT" or "ROLLBACK" */
@@ -705,6 +737,14 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 				return POOL_END;
 			}
 
+			/*
+			 * Check if some error detected.  If so, emit
+			 * log. This is usefull when invalid encoding error
+			 * occurs. In this case, PostgreSQL does not report
+			 1* what statement caused that error and make users
+			 * confused.
+			 */
+			per_node_error_log(backend, MASTER_NODE_ID, string, "SimpleQuery: Error or notice message from backend: ", true);
 
 			TSTATE(backend) = 'I';
 		}
@@ -712,8 +752,6 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 	}
 	else
 	{
-		free_parser();
-
 		per_node_statement_log(backend, MASTER_NODE_ID, string);
 
 		if (send_simplequery_message(MASTER(backend), len, string, MAJOR(backend)) != POOL_CONTINUE)
@@ -729,8 +767,20 @@ POOL_STATUS NotificationResponse(POOL_CONNECTION *frontend,
 				cancel_packet.key= MASTER_CONNECTION(backend)->key;
 				cancel_request(&cancel_packet);
 
+				free_parser();
 				return POOL_END;
 		}
+
+		/*
+		 * Check if some error detected.  If so, emit
+		 * log. This is usefull when invalid encoding error
+		 * occurs. In this case, PostgreSQL does not report
+		 1* what statement caused that error and make users
+		 * confused.
+		 */
+		per_node_error_log(backend, MASTER_NODE_ID, string, "SimpleQuery: Error or notice message from backend: ", true);
+
+		free_parser();
 	}
 
 	return POOL_CONTINUE;
@@ -1017,7 +1067,7 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 	len = ntohl(len) - 4;
 	string = pool_read2(frontend, len);
 
-	pool_debug("Parse: portal name <%s>", string);
+	pool_debug("Parse: statement name <%s>", string);
 
 	name = string;
 	stmt = string + strlen(string) + 1;
@@ -1071,6 +1121,11 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 		/* translate Parse message to PrepareStmt */
 		p_stmt = palloc(sizeof(PrepareStmt));
 		p_stmt->type = T_PrepareStmt;
+
+		/* XXX: there's a confusion here. Someone mixed up statement
+		 * name with portal name. It is regarded that statment name ==
+		 * portal name. Someday we should fix this. Sigh.
+		 */
 		p_stmt->name = pstrdup(name);
 		p_stmt->query = copyObject(node);
 		portal->stmt = (Node *)p_stmt;
@@ -1090,7 +1145,11 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 			pending_prepared_portal = portal;
 		}
 
-		/* switch old memory context */
+		/*
+		 * Switch to old memory context. Caution. Now we are in parser
+		 * memory context.
+		 * Palloced memories will be gone if free_parser() called!
+		 */
 		pool_memory = old_context;
 
 		if (REPLICATION)
@@ -1110,7 +1169,8 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 			rewrite_query = rewrite_timestamp(backend, node, rewrite_to_params, portal);
 			if (rewrite_query != NULL)
 			{
-				string = palloc(strlen(name) + strlen(rewrite_query) + 2);
+				int alloc_len = len - strlen(stmt) + strlen(rewrite_query);
+				string = palloc(alloc_len);
 				strcpy(string, name);
 				strcpy(string + strlen(name) + 1, rewrite_query);
 				memcpy(string + strlen(name) + strlen(rewrite_query) + 2,
@@ -1180,7 +1240,10 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 		return POOL_END;
 	}
 
-	free_parser();
+	/*
+	 * Cannot call free_parser() here. Since "string" might be allocated in parser context.
+	 * free_parser();
+	 */
 
 	if (REPLICATION || PARALLEL_MODE || MASTER_SLAVE)
 	{
@@ -1198,6 +1261,7 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 			cancel_packet.pid = MASTER_CONNECTION(backend)->pid;
 			cancel_packet.key= MASTER_CONNECTION(backend)->key;
 			cancel_request(&cancel_packet);
+			free_parser();
 			return POOL_END;
 		}
 
@@ -1209,7 +1273,20 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 		 */
 		deadlock_detected = detect_deadlock_error(MASTER(backend), MAJOR(backend));
 		if (deadlock_detected < 0)
+		{
+			free_parser();
 			return POOL_END;
+		}
+		else
+		{
+			/*
+			 * Check if other than dealock error detected.  If so, emit
+			 * log. This is usefull when invalid encoding error occurs. In
+			 * this case, PostgreSQL does not report what statement caused
+			 * that error and make users confused.
+			 */
+			per_node_error_log(backend, MASTER_NODE_ID, stmt, "Parse(): Error or notice message from backend: ", true);
+		}
 
 		for (i=0;i<NUM_BACKENDS;i++)
 		{
@@ -1225,7 +1302,10 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 												 strlen(POOL_ERROR_QUERY)+1,
 												 POOL_ERROR_QUERY,
 												 MAJOR(backend)))
+					{
+						free_parser();
 						return POOL_END;
+					}
 				}
 				else
 				{
@@ -1233,7 +1313,10 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 					per_node_statement_log(backend, i, per_node_statement_log_buffer);
 
 					if (send_extended_protocol_message(backend, i,"P", len, string))
+					{
+						free_parser();
 						return POOL_END;
+					}
 				}
 			}
 		}
@@ -1254,11 +1337,26 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend,
 				cancel_packet.pid = MASTER_CONNECTION(backend)->pid;
 				cancel_packet.key= MASTER_CONNECTION(backend)->key;
 				cancel_request(&cancel_packet);
+				free_parser();
 				return POOL_END;
 			}
+
+			/*
+			 * Check if error (or notice response) from backend is
+			 * detected.  If so, emit log. This is usefull when
+			 * invalid encoding error occurs. In this case, PostgreSQL
+			 * does not report what statement caused that error and
+			 * make users confused.
+			 */
+			per_node_error_log(backend, i, stmt, "Parse(): Error or notice message from backend: ", true);
 		}
 	}
 
+	/*
+	 * Ok. we are safe to call free_parser();
+	 */
+	free_parser();
+
 	for (;;)
 	{
 		POOL_STATUS ret;
@@ -1719,7 +1817,9 @@ POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend,
 			/* fall through */
 
 		default:
-			if (MAJOR(backend) == PROTO_MAJOR_V3)
+			if ((MAJOR(backend) == PROTO_MAJOR_V3) &&
+			    (fkind == 'S' || fkind == 'H' || fkind == 'D' || fkind == 'f'||
+				 fkind == 'C' || fkind == 'B' || fkind == 'F' || fkind == 'd' || fkind == 'c'))
 			{
 				if (MASTER_SLAVE &&
 					(TSTATE(backend) != 'I' || receive_extended_begin))
@@ -2678,7 +2778,7 @@ static int is_temp_table(POOL_CONNECTION_POOL *backend, Node *node)
 		}
 	}
 
-	hasrelistemp = (int)pool_search_relcache(hasrelistemp_cache, backend, "pg_class");
+	hasrelistemp = pool_search_relcache(hasrelistemp_cache, backend, "pg_class")==0?0:1;
 	if (hasrelistemp)
 		query = ISTEMPQUERY84;
 	else
@@ -2702,7 +2802,7 @@ static int is_temp_table(POOL_CONNECTION_POOL *backend, Node *node)
 	/*
 	 * Search relcache.
 	 */
-	result = (int)pool_search_relcache(relcache, backend, str);
+	result = pool_search_relcache(relcache, backend, str)==0?0:1;
 	return result;
 }
 
@@ -2716,3 +2816,19 @@ void per_node_statement_log(POOL_CONNECTION_POOL *backend, int node_id, char *qu
 	if (pool_config->log_per_node_statement)
 		pool_log("DB node id: %d backend pid: %d statement: %s", node_id, ntohl(slot->pid), query);
 }
+
+/*
+ * Check kind and produce error message
+ * All data read in this function is returned to stream.
+ */
+void per_node_error_log(POOL_CONNECTION_POOL *backend, int node_id, char *query, char *prefix, bool unread)
+{
+	POOL_CONNECTION_POOL_SLOT *slot = backend->slots[node_id];
+	char *message;
+
+	if (pool_extract_error_message(true, CONNECTION(backend, node_id), MAJOR(backend), true, &message) > 0)
+	{
+		pool_log("%s: DB node id: %d backend pid: %d statement: %s message: %s",
+				 prefix, node_id, ntohl(slot->pid), query, message);
+	}
+}
