@@ -22,6 +22,10 @@ public class Timestamp extends PgpoolTest {
 	    pstmt.executeUpdate();
 	    pstmt.close();
 
+	    pstmt = connection.prepareStatement("SELECT * FROM t1");
+	    pstmt.executeQuery();
+	    pstmt.close();
+
 	    connection.commit();
 	}
 	finally {
